# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/uom-project',
                'doc_host': 'https://AugustasMacijauskas.github.io',
                'git_url': 'https://github.com/AugustasMacijauskas/uom-project',
                'lib_path': 'uom_project'},
  'syms': { 'uom_project.core': { 'uom_project.core.calculate_force': ('core.html#calculate_force', 'uom_project/core.py'),
                                  'uom_project.core.calculate_velocity': ('core.html#calculate_velocity', 'uom_project/core.py'),
                                  'uom_project.core.lu_solve': ('core.html#lu_solve', 'uom_project/core.py'),
                                  'uom_project.core.setup_poisson_problem': ('core.html#setup_poisson_problem', 'uom_project/core.py'),
                                  'uom_project.core.solve_sparse_linear_system': ( 'core.html#solve_sparse_linear_system',
                                                                                   'uom_project/core.py'),
                                  'uom_project.core.solve_sparse_linear_system_jax': ( 'core.html#solve_sparse_linear_system_jax',
                                                                                       'uom_project/core.py'),
                                  'uom_project.core.solve_sparse_linear_system_pytorch': ( 'core.html#solve_sparse_linear_system_pytorch',
                                                                                           'uom_project/core.py')},
            'uom_project.poisson_solvers': { 'uom_project.poisson_solvers.construct_laplacian_kernel_matrix': ( 'poisson_solvers.html#construct_laplacian_kernel_matrix',
                                                                                                                'uom_project/poisson_solvers.py'),
                                             'uom_project.poisson_solvers.f': ('poisson_solvers.html#f', 'uom_project/poisson_solvers.py'),
                                             'uom_project.poisson_solvers.get_jacobian': ( 'poisson_solvers.html#get_jacobian',
                                                                                           'uom_project/poisson_solvers.py'),
                                             'uom_project.poisson_solvers.newton_iterator': ( 'poisson_solvers.html#newton_iterator',
                                                                                              'uom_project/poisson_solvers.py'),
                                             'uom_project.poisson_solvers.poisson_gauss_seidel_with_sor_solver': ( 'poisson_solvers.html#poisson_gauss_seidel_with_sor_solver',
                                                                                                                   'uom_project/poisson_solvers.py'),
                                             'uom_project.poisson_solvers.poisson_newton_alternative_solver': ( 'poisson_solvers.html#poisson_newton_alternative_solver',
                                                                                                                'uom_project/poisson_solvers.py'),
                                             'uom_project.poisson_solvers.poisson_newton_solver': ( 'poisson_solvers.html#poisson_newton_solver',
                                                                                                    'uom_project/poisson_solvers.py'),
                                             'uom_project.poisson_solvers.poisson_non_iterative_solver': ( 'poisson_solvers.html#poisson_non_iterative_solver',
                                                                                                           'uom_project/poisson_solvers.py')},
            'uom_project.streamfunction_vorticity': { 'uom_project.streamfunction_vorticity.get_dw_dt': ( '02_streamfunction_vorticity_iterative_solver/streamfunction_vorticity_iterative_solver.html#get_dw_dt',
                                                                                                          'uom_project/streamfunction_vorticity.py'),
                                                      'uom_project.streamfunction_vorticity.streamfunction_vorticity_iterative_solver': ( '02_streamfunction_vorticity_iterative_solver/streamfunction_vorticity_iterative_solver.html#streamfunction_vorticity_iterative_solver',
                                                                                                                                          'uom_project/streamfunction_vorticity.py'),
                                                      'uom_project.streamfunction_vorticity.update_vorticity_bcs': ( '02_streamfunction_vorticity_iterative_solver/streamfunction_vorticity_iterative_solver.html#update_vorticity_bcs',
                                                                                                                     'uom_project/streamfunction_vorticity.py')},
            'uom_project.streamfunction_vorticity_jax': { 'uom_project.streamfunction_vorticity_jax.f_jax': ( 'streamfunction_vorticity_jax.html#f_jax',
                                                                                                              'uom_project/streamfunction_vorticity_jax.py'),
                                                          'uom_project.streamfunction_vorticity_jax.newton_iteration_step': ( 'streamfunction_vorticity_jax.html#newton_iteration_step',
                                                                                                                              'uom_project/streamfunction_vorticity_jax.py'),
                                                          'uom_project.streamfunction_vorticity_jax.newton_iterator_jax': ( 'streamfunction_vorticity_jax.html#newton_iterator_jax',
                                                                                                                            'uom_project/streamfunction_vorticity_jax.py'),
                                                          'uom_project.streamfunction_vorticity_jax.newton_solver_jax': ( 'streamfunction_vorticity_jax.html#newton_solver_jax',
                                                                                                                          'uom_project/streamfunction_vorticity_jax.py'),
                                                          'uom_project.streamfunction_vorticity_jax.reconstruct_w_jax': ( 'streamfunction_vorticity_jax.html#reconstruct_w_jax',
                                                                                                                          'uom_project/streamfunction_vorticity_jax.py')},
            'uom_project.streamfunction_vorticity_newton': { 'uom_project.streamfunction_vorticity_newton.f': ( 'streamfunction_vorticity_newton.html#f',
                                                                                                                'uom_project/streamfunction_vorticity_newton.py'),
                                                             'uom_project.streamfunction_vorticity_newton.get_jacobian': ( 'streamfunction_vorticity_newton.html#get_jacobian',
                                                                                                                           'uom_project/streamfunction_vorticity_newton.py'),
                                                             'uom_project.streamfunction_vorticity_newton.get_standard_basis_vector': ( 'streamfunction_vorticity_newton.html#get_standard_basis_vector',
                                                                                                                                        'uom_project/streamfunction_vorticity_newton.py'),
                                                             'uom_project.streamfunction_vorticity_newton.newton_iterator': ( 'streamfunction_vorticity_newton.html#newton_iterator',
                                                                                                                              'uom_project/streamfunction_vorticity_newton.py'),
                                                             'uom_project.streamfunction_vorticity_newton.newton_solver': ( 'streamfunction_vorticity_newton.html#newton_solver',
                                                                                                                            'uom_project/streamfunction_vorticity_newton.py'),
                                                             'uom_project.streamfunction_vorticity_newton.reconstruct_w': ( 'streamfunction_vorticity_newton.html#reconstruct_w',
                                                                                                                            'uom_project/streamfunction_vorticity_newton.py')},
            'uom_project.streamfunction_vorticity_pytorch': { 'uom_project.streamfunction_vorticity_pytorch.f_pytorch': ( 'streamfunction_vorticity_pytorch.html#f_pytorch',
                                                                                                                          'uom_project/streamfunction_vorticity_pytorch.py'),
                                                              'uom_project.streamfunction_vorticity_pytorch.newton_iterator_pytorch': ( 'streamfunction_vorticity_pytorch.html#newton_iterator_pytorch',
                                                                                                                                        'uom_project/streamfunction_vorticity_pytorch.py'),
                                                              'uom_project.streamfunction_vorticity_pytorch.newton_solver_pytorch': ( 'streamfunction_vorticity_pytorch.html#newton_solver_pytorch',
                                                                                                                                      'uom_project/streamfunction_vorticity_pytorch.py'),
                                                              'uom_project.streamfunction_vorticity_pytorch.reconstruct_w_pytorch': ( 'streamfunction_vorticity_pytorch.html#reconstruct_w_pytorch',
                                                                                                                                      'uom_project/streamfunction_vorticity_pytorch.py')}}}
